import glob
import re
import requests
from requests.exceptions import ConnectionError
from requests.exceptions import ConnectTimeout
import xlwt
import xlrd
from xlutils.copy import copy
from xlwt.Style import default_style
import platform
import random
from bottle import template
import webbrowser
import sys
import datetime
import logging
import os
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from email.header import Header


def logger():
    class LoggerHandler:
        _logger_level = {'debug': logging.DEBUG, 'info': logging.INFO, 'warning': logging.WARNING,
                         'error': logging.ERROR, 'critical': logging.CRITICAL}

        def __init__(self, log_name, file_name, logger_level, stream_level='info', file_level='warning'):
            self.log_name, self.file_name, self.logger_level, self.stream_level, self.file_level = log_name, file_name, self._logger_level.get(
                logger_level, 'debug'), self._logger_level.get(stream_level, 'info'), self._logger_level.get(file_level,
                                                                                                             'warning')
            self.logger = logging.getLogger(self.log_name)
            self.logger.setLevel(self.logger_level)
            if not self.logger.handlers:
                f_stream, f_file = logging.StreamHandler(), logging.FileHandler(self.file_name, encoding='utf-8')
                f_stream.setLevel(self.stream_level)
                f_file.setLevel(self.file_level)

                # set log output stream, or"%(asctime)s %(name)s %(levelname)s %(message)s", see more setting in "formater"
                formatter = logging.Formatter("%(asctime)s %(levelname)s %(message)s")
                f_stream.setFormatter(formatter)
                f_file.setFormatter(formatter)
                self.logger.addHandler(f_stream)
                self.logger.addHandler(f_file)

        @property
        def get_logger(self):
            return self.logger

    log_folder = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'logs')
    log_file_name = os.path.join(log_folder, datetime.datetime.now().strftime('%Y-%m-%d') + '.log')
    if not os.path.exists(log_folder):
        os.mkdir(log_folder)
    return LoggerHandler(log_name="DEFAULT", logger_level='debug', file_name=log_file_name, stream_level='debug',
                         file_level='info').get_logger


log = logger()


class Api:
    def __init__(self, excel_file: str, sheet=0, mark=1, is_email=False):
        self.TIME1 = datetime.datetime.now()
        excel_file = os.path.abspath(excel_file)
        folder = os.path.dirname(excel_file)
        self.folder = folder if folder else print('no such file %s' % excel_file) or sys.exit()
        log.info('start...')
        self.excel_file, self.req, self.excel = excel_file, Request(), ApiExcel(excel_file)
        self.headers = self.excel.get_header()
        mark, sheet = mark if isinstance(mark, int) else 1, sheet if isinstance(sheet, int or str) else 0
        self.rows, self.mark, self.is_email = self.excel.get_rows(sheet=sheet), mark, is_email
        self.run()

    def run(self):
        all_cases, pass_cases, fail_cases, fake_fail_cases = self.do_req()
        self.do_res(all_cases)  # for excel
        Report.generate(all_cases, pass_cases, fail_cases, fake_fail_cases, report_file_folder=self.folder,
                        waste_time=datetime.datetime.now() - self.TIME1)  # for html
        self.do_email(self.is_email)

    def do_req(self):
        # Get initial information
        e, status_code_li, all_cases, pass_cases, fail_cases, fake_fail_cases, res_flag = self.excel, [], [], [], [], [], '接口'

        # Traverse all rows
        for i in range(1, self.rows):
            if e.get_run(i) != self.mark:
                continue

            method, url, data = e.get_method(i).lower(), e.get_url(i), e.get_req_data(i)
            data = data if data else None
            rely = self.excel.get_rely_express(i)
            if rely:
                if not data or '{}' not in data:
                    res = 'the data of rely row %s is invalid' % (i + 1)
                    log.error(res)
                    status_code, res_text, res_flag = 402, 'Excel value error: ' + res, '内部'
                else:
                    data = data.replace("{}", '%s')
                    extract = self.get_extract(rely)
                    if extract:
                        data = data % extract
                        status_code, res_text = self.send_req(method, url, data, self.headers)
                    else:
                        status_code, res_text, res_flag = 402, 'get None from rely extract', '内部'

            else:
                status_code, res_text = self.send_req(method, url, data, self.headers)
                res_text = res_text[:Setting.MAX_LENGTH]
            res_text = res_text if len(res_text) < Setting.MAX_LENGTH else res_text + " ..."

            # analysis result
            pre_fail, expect = e.get_pre_fail(i), e.get_assert(i)
            if not expect:
                if status_code == 200:
                    result, flag = 'PASS', 1
                elif pre_fail:
                    result, flag = 'PASS', 2
                else:
                    result, flag = 'FAIL', 0
            else:
                if expect in res_text:
                    result, flag = 'PASS', 1
                elif pre_fail:
                    result, flag = 'PASS', 2
                else:
                    result, flag = 'FAIL', 0

            status_code_li.append(status_code)

            req_data, detail_request_url = data if data else '', '请求地址：' + url + '\n'
            detail_request_data, detail_response = '请求参数：' + req_data + '\n', res_flag + '响应：' + res_text + '\n'

            # Avoid interpreting labels, for html
            detail, explain = detail_request_url + detail_request_data + detail_response.replace('<', '&lt;').replace('>', '&gt;'), e.get_explain(i)
            the_case = [str(i), explain, url[:Setting.URL_CUT] + '...' if len(url) > Setting.URL_CUT else url, status_code, expect, result, '详细', detail]
            all_cases.append(the_case)
            fail_cases.append(the_case) if flag == 0 else fake_fail_cases.append(
                the_case) if flag == 2 else pass_cases.append(the_case)
        return all_cases, pass_cases, fail_cases, fake_fail_cases

    def send_req(self, method: str, url: str, data: str, headers: dict):
        req = self.req
        try:
            if method == "get":
                status_code, res_text = req.get(url, params=data, headers=headers)
            elif method in ["post", "put"]:
                data = eval(data)
                if not type(data) == dict:
                    status_code, res_text = 402, "invalid request data that can't convert to dict"
                else:
                    status_code, res_text = req.request(method, url, data=data, headers=headers)
            elif method == 'delete':
                status_code, res_text = req.delete(url, data=data, headers=headers)
            else:
                raise ValueError("must be get/post/put/delete")
        except (ConnectTimeout, ConnectionError):
            status_code, res_text = 500, "ConnectTimeout"
        return status_code, res_text

    def get_extract(self, rely: tuple):
        """
        get rely response information by RE
        """
        e = self.excel
        rely_row, rely_express = rely
        rely_row = int(rely_row) - 1
        rely_method, rely_url, rely_data = e.get_method(rely_row), e.get_url(rely_row), e.get_req_data(rely_row)
        rely_status_code, rely_res_text = self.send_req(rely_method, rely_url, rely_data, self.headers)
        is_ = re.search(rely_express, rely_res_text)
        before_index = rely_express.rindex("\\") - 1
        before_index = before_index if before_index > 0 else 0
        return is_.group()[before_index:] if is_ else None

    def do_res(self, case_li: list):
        success_num = fail_num = 0
        fail_urls, fail_status = [], []
        for _ in case_li:
            id_, explain, url, s_code, expect, result, no_use, detail = _
            case_depend = self.excel.get_case_depend(id_)
            star = detail.index('响应：') + 3
            res = detail[star:]
            if case_depend not in Setting.DEPENDED_FLAG:
                res = detail[star:Setting.CUT_RANGE]

            self.excel.write_status_code(id_, s_code)
            self.excel.write_response(id_, res)
            if result == 'PASS':
                log.info('%d \t%s' % (s_code, url))
                success_num += 1
                color_index = 3
            else:
                log.error('%d \t%s' % (s_code, url))
                color_index = 4
                fail_num += 1

            self.excel.write_result(id_, result, color_index)

        all_num = success_num + fail_num
        all_num = all_num if all_num else 1

        # success rate and failure rate
        success_rate, failure_rate = round(success_num / all_num, 4) * 100, round(fail_num / all_num, 4) * 100

        log.info("执行完毕...\n\n本次共执行 %d条用例，通过%d条，失败%d条；成功率为%d%%, 失败率为%d%%" % (
            all_num, success_num, fail_num, success_rate, failure_rate))
        return success_num, fail_num, fail_urls, fail_status

    def do_email(self, is_email):
        if not is_email:
            return

        email_dict = self.excel.email_sheet()

        # weather run
        is_email2 = email_dict.get('run')
        if not is_email2 or is_email2 in Setting.FLAG_NO:
            log.info("the email run is an empty string, do not send email")
            return
        elif is_email2 not in Setting.FLAG_YES:
            log.error("ValueError: the email run must in %s" % (Setting.FLAG_YES + Setting.FLAG_NO))
            return

        # msg for email
        name, pd, to, tittle, content, attachment = email_dict.get('name'), email_dict.get('password'), email_dict.get(
            'to'), email_dict.get('tittle'), email_dict.get('content'), email_dict.get('attachment')

        if not (name and pd and to):
            log.error("email name or password missing")
            return False

        if attachment and '.' not in attachment:
            log.error("wrong file type %s" % attachment)
            return

        return Email(name, pd, to, tittle, content, attachment)


class Excel:
    def __init__(self, excel_file, sheet_name=''):
        self.excel_file = excel_file
        self.book = xlrd.open_workbook(excel_file, formatting_info=True)

        # open the table with the specified name
        try:
            if sheet_name:
                self.sheet_ = self.book.sheet_by_name(sheet_name=sheet_name)
                self.sheet_name_index = self.book.sheet_names().index(sheet_name)
            else:
                self.sheet_ = self.book.sheet_by_index(0)
                self.sheet_name_index = 0
        except xlrd.biffh.XLRDError:
            print("no such table %s' in excel" % sheet_name)
        self.book_copy = copy(self.book)
        self.sheet = self.book_copy.get_sheet(self.sheet_name_index)

    def cell_value(self, row: int, col):
        value = self.sheet_.cell_value(int(row), col)
        return value

    def get_run(self, row: int):
        run = self.cell_value(row, Setting.RUN)
        try:
            run = int(run)
        except ValueError:
            print("RUN must be a number")
        return run

    def get_explain(self, row: int):
        return self.cell_value(row, Setting.EXPLAIN)

    def get_method(self, row: int):
        method = self.cell_value(row, Setting.METHOD)
        if method.lower() not in ["get", "post", "put", "delete"]:
            raise ValueError("method invalid")
        return method

    def get_url(self, row: int):
        url = self.cell_value(row, Setting.URL)
        if not url.startswith("http"):
            raise ValueError("url invalid")
        return url

    def get_req_data(self, row: int):
        return self.cell_value(row, Setting.REQ_DATA)

    def get_status_code(self, row: int):
        return self.cell_value(row, Setting.STATUS_CODE)

    def get_response(self, row):
        return self.cell_value(row, Setting.RESPONSE)

    def get_case_depend(self, row: int):
        row = int(row)
        case_depend = self.cell_value(row, Setting.CASE_DEPEND)
        if not case_depend:
            return False

        if case_depend in Setting.DEPENDED_FLAG:
            return case_depend  # it might be '!' or '', that means be relied

        try:
            case_depend = int(float(case_depend))
        except ValueError:
            raise ValueError(("case depend must be int or !, the wrong row %s " % row))

        if case_depend > row:
            raise ValueError("warn: depended row %s must at the top of row %s" % (row, case_depend))
        return case_depend

    def get_rely_express(self, row):
        rely_row = self.get_case_depend(row)
        if rely_row:
            return rely_row, self.cell_value(row, Setting.RELY_EXPRESS)

    def get_assert(self, row):
        return self.cell_value(row, Setting.ASSERT)

    def get_pre_fail(self, row):
        pre_fail = self.cell_value(row, Setting.PRE_FAIL)
        if pre_fail:
            if pre_fail not in Setting.FLAG_YES:
                raise ValueError("pre fail invalid, it must in %s" % Setting.FLAG_YES)
        return pre_fail

    def get_result(self, row):
        return self.cell_value(row, Setting.RESULT)

    def get_columns(self, sheet='', row=0):
        return self.all_columns(sheet=sheet, row=row)

    def get_rows(self, sheet=0):
        return self.all_rows(sheet=sheet)

    def get_email(self):
        return self.email_sheet()

    def get_header(self):
        return self.header_sheet()


class ApiExcel(Excel):
    def email_sheet(self) -> dict:
        if "email" not in self.book.sheet_names():
            print("no such sheet 'email'")
            return {}
        es = self.book["email"]
        cols = self.all_columns(es)
        values = self.all_columns(es, row=1)
        email_dict = {}
        for i, j in zip(cols, values):
            email_dict[i] = j
        return email_dict

    def header_sheet(self):
        if "header" not in self.book.sheet_names():
            print("no such sheet 'header'")
            return None
        hs = self.book["header"]
        cols = self.get_columns(hs)
        values = self.get_columns(hs, row=1)
        headers = {}
        for i, j in zip(cols, values):
            headers[i] = j
        return headers

    def all_columns(self, sheet='', row=0):
        columns = []
        sheet = sheet if sheet else self.sheet_
        try:
            for i in range(20):
                col = sheet.cell_value(row, i)
                if col:
                    columns.append(col)
        except IndexError:
            pass
        return columns

    def all_rows(self, sheet=0):
        sheet = sheet if sheet else self.sheet_
        return sheet.nrows

    @staticmethod
    def gen_style(cell_style=1):
        """
        Beautification of Excel form style
        """
        style, al, font = xlwt.XFStyle(), xlwt.Alignment(), xlwt.Font()
        if cell_style == 1:
            style.alignment.wrap = 1  # wrap automatically
        elif cell_style == 2:
            style.alignment.wrap = 1  # wrap automatically
            font.colour_index = 4
        elif cell_style == 3:
            font.bold = True
            font.colour_index = 3  # set green color
        elif cell_style == 4:
            font.bold = True
            font.colour_index = 2  # set red color
        else:
            style.alignment.wrap = 1
        style.alignment = al
        style.font = font
        style.alignment.vert = 0x01  # set vertical center
        style.alignment.horz = 0x02  # set horizontal center
        # to see the details by "print(style.alignment.vert, style.font.bold)"
        return style

    def write_cell(self, row, col, value: str, style=default_style):
        self.sheet.write(int(row), col, str(value), style)
        self.book_copy.save(self.excel_file)

    def write_response(self, row, value):
        """
        write response to excel
        """
        return self.write_cell(row, Setting.RESPONSE, value)

    def write_status_code(self, row, status_code):
        row = int(row)
        if status_code == 200:
            color = 3
        else:
            color = 4
        return self.write_cell(row, Setting.STATUS_CODE, status_code, self.gen_style(color))

    def write_result(self, row, result, color_index: int):
        return self.write_cell(row, Setting.RESULT, result, self.gen_style(color_index))


class Request:
    def request(self, method: str, url: str, data=None, json=None, **kwargs):
        res = requests.request(method=method, url=url, data=data, json=json, **kwargs)
        print(method, url, data, kwargs)
        return self.call_back(res)

    def get(self, url: str, params=None, **kwargs):
        res = requests.get(url=url, params=params, **kwargs)
        return self.call_back(res)

    def delete(self, url: str, data=None, **kwargs):
        res = requests.delete(url=url, data=data, **kwargs)
        return self.call_back(res)

    @staticmethod
    def call_back(res):
        status_code = res.status_code
        res_text = res.text
        return status_code, res_text


cases = {}


class Report:
    @staticmethod
    def generate(all_case: list = '', right_case: list = '', error_case: list = '',
                 untreaded_cases: list = '', report_file_folder='', waste_time=None):
        """
        generate report by all these cases
        """
        global cases
        if not all_case or (not right_case and not error_case and not untreaded_cases):
            raise ValueError("No cases report")

        cases = {'all_cases': all_case,
                 'right_cases': right_case,
                 'error_cases': error_case,
                 'untreaded_cases': untreaded_cases}

        # generate report
        html_report_object = HTML_REPORT(report_cases=cases, delta=waste_time)
        html = html_report_object.gen_html_report(Setting.html_)

        # saved file
        if not report_file_folder:
            report_file_folder = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        report_file_folder_save = os.path.join(report_file_folder, "reports")
        if not os.path.exists(report_file_folder_save):
            os.mkdir(report_file_folder_save)
        with open(os.path.join(report_file_folder_save,
                               datetime.datetime.now().strftime("%Y%m%d_%H%M%S") + ".html"), 'wb') as f:
            f.write(html.encode('utf-8'))

        # fixed file
        fixed_path = os.path.join(report_file_folder, "report.html")
        with open(fixed_path, 'wb') as f:
            f.write(html.encode('utf-8'))

        # Keep only the report of the latest (month)
        for i in glob.glob(report_file_folder_save + os.sep + '*.html')[:-30]:
            os.remove(i)
        if platform.system().lower() in ["windows", "macos"]:
            webbrowser.open(fixed_path)


class HTML_REPORT:
    def __init__(self, html_title: str = '测试报告', pie_theme: str = '数据统计', test_info: list = None,
                 report_table_title: list = None, report_cases: dict = None, delta=None):
        self.html_title = html_title
        self.pie_theme = pie_theme
        if not pie_theme or not isinstance(html_title, str):
            print('invalid html title or pie name')
            sys.exit(0)

        if not report_table_title:
            self.report_table_title = ['用例id', '接口说明', 'URL', '状态码', '期望值', '测试结果', '详情']
        elif not isinstance(report_table_title, list):
            print('invalid type of report_table_title, it must be list')
            sys.exit(0)

        now1 = datetime.datetime.now()
        time1 = now1.strftime("%Y-%m-%d %H:%M:%S")
        if not delta:
            m, s = random.randint(0, 3), random.randint(1, 59)
            delta = datetime.timedelta(seconds=s, minutes=m)
        time2 = (now1 - delta).strftime("%Y-%m-%d %H:%M:%S")
        d_s = delta.seconds + 1
        m, s = d_s // 60, d_s % 60
        waste = str(m) + "分" + str(s) + "秒"

        len_all, len_r, len_e, len_u = len(report_cases), len(report_cases['right_cases']), len(
            report_cases['error_cases']), len(report_cases['untreaded_cases'])

        if not test_info:
            test_info = [
                time2, time1, waste, len(report_cases['all_cases']),
                len_r + len_e, len_u]
            self.test_info = test_info
        elif not isinstance(test_info, list):
            print('invalid type of test_info, it must be list')
            sys.exit(0)

        if isinstance(report_cases, dict) and len_all > 0:
            self.report_cases = report_cases
            self.pie_sum_number = {
                'right_sum': len_r,
                'error_sum': len_e,
                'untreated_sum': len_u}
        else:
            print('invalid type of report_cases, it must be list')
            sys.exit(0)

    def packaged_cases(self, export_label_title, data_key, class_name, pannel_num):
        cases_num, cases_packaged = 1, ''
        cases_packaged = ''.join(
            [cases_packaged, '                   <tr class="{}">\n'.format(class_name), export_label_title,
             '                   </tr>\n'])
        for data_case in self.report_cases[data_key]:
            detail_id, hidden_id = ''.join([pannel_num, '-detail-', str(cases_num)]), ''.join(
                [pannel_num, '-hidden-', str(cases_num)])
            cases_num += 1
            cases_packaged = ''.join([cases_packaged, '                   <tr class="{}">\n'.format(class_name)])
            for data in data_case[:-1]:
                if '详细' == data:
                    cases_packaged = ''.join([cases_packaged,
                                              Setting.table_detail.format(detail_id, hidden_id, 'success',
                                                                          detail_id)])
                else:
                    cases_packaged = ''.join([cases_packaged, Setting.table_data.format(data)])
            cases_packaged = ''.join([cases_packaged, '                   </tr>\n'])
            cases_packaged = ''.join(
                [cases_packaged, Setting.detail_text.format(hidden_id, len(data_case) - 1, data_case[-1])])
        return cases_packaged

    def gen_html_report(self, html_template):
        export_label_title = ''
        for label_title in self.report_table_title:
            export_label_title = ''.join([export_label_title, Setting.title.format(label_title)])

        str_right_datas, str_untreated_cases, str_error_cases, str_all_cases = '', '', '', ''
        all_num = len(cases['all_cases'])
        right_num = len(cases['right_cases'])
        rate = '{:.2%}'.format(round(right_num / all_num, 5))

        for data_key in self.report_cases.keys():
            if 'right_cases' == data_key:
                str_right_datas = self.packaged_cases(export_label_title, data_key, 'success', 'panel1')
            elif 'untreaded_cases' == data_key:
                str_untreated_cases = self.packaged_cases(export_label_title, data_key, 'untreaded', 'panel3')
            elif 'error_cases' == data_key:
                str_error_cases = self.packaged_cases(export_label_title, data_key, 'error', 'panel2')
            else:
                str_all_cases = self.packaged_cases(export_label_title, data_key, 'all', 'panel0')
        export_label_datas = Setting.temp_label.format(
            all_num, right_num,
            len(cases['error_cases']),
            len(cases['untreaded_cases']),
            str(str_all_cases[0:-1]),
            str(str_right_datas[0:-1]),
            str(str_error_cases[0:-1]),
            str(str_untreated_cases[0:-1])
        )
        # right_name_and_rate = '成功 ' + rate
        html_template = ''.join([html_template, export_label_datas])
        view = template(
            html_template,
            html_title=self.html_title,
            theme=self.pie_theme,
            start_time=self.test_info[0],
            # end_time=self.test_info[1],
            used_time=self.test_info[2],
            rate=rate,
            sum_all_cases=self.test_info[3],
            sum_executed_cases=self.test_info[4],
            sum_untreaded_cases=self.test_info[5],
            right_sum=self.pie_sum_number['right_sum'],
            error_sum=self.pie_sum_number['error_sum'],
            untreated_sum=self.pie_sum_number['untreated_sum']
        )
        return view


class Email:
    def __init__(self, email_account: str, email_password: str, email_to: str,
                 email_title='', email_content='', file_list=None):
        """
        a simple send email method
        """
        server = smtplib.SMTP('smtp.qq.com')
        message = MIMEMultipart()  # email body
        print("sending email...")
        email_title = email_title if email_title else "Test Report!"
        email_content = email_content if email_content else 'Your test result has been generated. Please check the attachment for details!\n'

        # set email
        message['From'] = Header(f"<{email_account}>", 'utf-8')
        message['Subject'] = Header(email_title, 'utf-8')
        message.attach(MIMEText(email_content))

        if file_list:
            for file in file_list:
                try:
                    file_apart = MIMEApplication(open(file, 'rb').read(), file.split('.')[-1])
                    file_apart.add_header('Content-Disposition', 'attachment', filename=file.split("\\")[-1])
                    message.attach(file_apart)
                except Exception as e:
                    print("wrong file %s %s" % (file, str(e)))

        # send email to someone
        try:
            server.login(email_account, email_password)
            server.sendmail(email_account, email_to, message.as_string())
            print("Mail sent successfully! addressee:" + str(email_to))
            server.quit()
        except smtplib.SMTPException as e:
            print('Mail sending failed! Error message:' + str(e))


class Setting:
    """--------------corresponding to the columns in Excel---------------------"""
    RUN = 0
    EXPLAIN = 1
    METHOD = 2
    URL = 3
    REQ_DATA = 4
    STATUS_CODE = 5
    RESPONSE = 6
    CASE_DEPEND = 7  # =ROW(E3)
    RELY_EXPRESS = 8  # jsonpath
    ASSERT = 9
    PRE_FAIL = 10
    RESULT = 11

    """--------------corresponding to the columns in Excel---------------------"""

    """---------------do not change the configuration easily-------------------"""
    URL_CUT = 40
    CUT_RANGE = 1000  # how many word to save, for excel
    MAX_LENGTH = 6666  # the max of xlwt to excel is 65536, but consuming too much memory while all in the max, It's better to be one order of magnitude less
    """---------------do not change the configuration easily-------------------"""

    # excel flag
    FLAG_NO = ['N', 'n', 'No', 'NO', 'no', '0']
    FLAG_YES = ['Y', 'y', 'Yes', 'YES', '1']
    DEPENDED_FLAG = ['!', '！']

    """----------------for html------------------------------------------------"""
    html_ = u"""
    <?xml version="1.0" encoding="UTF-8"?>
    <html>
    <head>
        <title>{{html_title}}</title>
        <meta name="generator" content="HTMLTestRunner 0.8.2.2"/>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>

        
        <link href="https://libs.baidu.com/bootstrap/3.0.3/css/bootstrap.min.css" rel="stylesheet">
        <script src="https://libs.baidu.com/jquery/2.0.0/jquery.min.js"></script>
        <script src="https://libs.baidu.com/bootstrap/3.0.3/js/bootstrap.min.js"></script>
        <script src="https://cdn.bootcss.com/echarts/3.7.1/echarts.min.js"></script>
        

        <!-- 引入js
        <link href="js/bootstrap.min.css" rel="stylesheet">
        <script src="js/jquery.min.js"></script>
        <script src="js/bootstrap.min.js"></script>
        <script src="js/echarts.js"></script>
         -->

        <style type="text/css" media="screen">
            body {
                margin: 0;
                font-family: "Arial", "Microsoft YaHei", "黑体", "宋体", sans-serif;
                font-size: 18px;
                line-height: 1.5;
                line-height: 1.5;
                color: #333333;
            }

            .table {
                margin-bottom: 1px;
                width: 100%;
            }

            .hiddenRow {
                display: none;
            }

            .container-fluid {
                padding-right: 120px;
                padding-left: 120px;
            }

            .nav-tabs li {
                width: 186px;
                text-align: center;
            }
        </style>
    </head>

    <body >
        <script language="javascript" type="text/javascript">

        function showClassDetail(detail_id, hiddenRow_id, class_type) {
            console.log(document.getElementById(hiddenRow_id).className)

            if ('详细' ==  document.getElementById(detail_id).innerText) {
                if ('all' == class_type) {
                    document.getElementById(hiddenRow_id).className = 'all';
                }
                else if ('success' == class_type) {
                    document.getElementById(hiddenRow_id).className = 'success';
                }
                else if ('error' == class_type) {
                    document.getElementById(hiddenRow_id).className = 'error';
                }
                else{
                    document.getElementById(hiddenRow_id).className = 'untreaded';
                }
                document.getElementById(detail_id).innerText = "收起"
            }
            else {
                document.getElementById(detail_id).innerText = "详细"
                document.getElementById(hiddenRow_id).className = 'hiddenRow';
            }
        }

        </script>

        <div class="container-fluid">
            <div class="page-header">
                <h1 class="text-primary" style="font-size:45px;line-height:75px">&nbsp;&nbsp;{{html_title}}</h1>
            </div>

            <div class="col-md-12">
                <div class="col-md-4" style="Background-Color:#F5F5F5; height:300px">
                    <h3 style="line-height:25px">基本信息</h3>
                    <table class="table table-hover table-bordered" style="width:100%; height:11px">
                        <tbody>
                            <tr class="info">
                                <td class="text-center">开始时间</td>
                                <td class="text-center">{{start_time}}</td>
                            </tr>
                            <tr class="info">
                                <td class="text-center">测试耗时</td>
                                <td class="text-center">{{used_time}}</td>
                            </tr>
                            <tr class="info">
                                <td class="text-center">接口成功率</td>
                                <td class="text-center">{{rate}}</td>
                            </tr>
                            <tr class="info">
                                <td class="text-center">总用例个数</td>
                                <td class="text-center">{{sum_all_cases}}</td>
                            </tr>
                            <tr class="info">
                                <td class="text-center">执行用例数</td>
                                <td class="text-center">{{sum_executed_cases}}</td>
                            </tr>
                            <tr class="info">
                                <td class="text-center">跳过用例数</td>
                                <td class="text-center">{{sum_untreaded_cases}}</td>
                            </tr>
                        </tbody>
                    </table>
                </div>

                <div class="col-md-8">
                    <!-- 为ECharts准备一个具备大小（宽高）的Dom -->
                    <div id="main" style="height:300px;"></div>
                    <script type="text/javascript">
                        var myChart = echarts.init(document.getElementById('main'));
                        var option = {
                        backgroundColor: '#F5F5F5', //背景色

                        title: {
                            text: '数据统计',
                            x: 'center'
                        },

                        legend: {
                            orient: 'vertical',
                            x: 'left',
                            data: ['成功', '失败', '未检验']
                        },

                        color: ['#3c763d', '#a94442', '#0099CC'],

                        calculable: true,

                        series: [{
                            name: '测试结果',
                            type: 'pie',
                            radius: '55%',
                            center: ['50%', '60%'],
                            startAngle: 135,
                            data: [{
                                value: {{right_sum}},
                                name: '成功',
                                itemStyle: {
                                    normal: {
                                        label: {
                                            formatter: '{b} : {c} ({d}%)',
                                            textStyle: {
                                                align: 'left',
                                                fontSize: 15,
                                            }
                                        },
                                        labelLine: {
                                             length: 40,
                                        }
                                     }
                                }
                            }, {
                                value: {{error_sum}},
                                name: '失败',
                                itemStyle: {
                                    normal: {
                                        label: {
                                            formatter: '{b} : {c} ({d}%)',
                                            textStyle: {
                                                align: 'right',
                                                fontSize: 15,
                                            }
                                        },
                                        labelLine: {
                                            length: 40,
                                            }
                                        }
                                    }
                                }, {
                                value: {{untreated_sum}},
                                name: '未检验',
                                itemStyle: {
                                    normal: {
                                        label: {
                                            formatter: '{b} : {c} ({d}%)',
                                            textStyle: {
                                                align: 'right',
                                                fontSize: 15,
                                            }
                                       },
                                        labelLine: {
                                            length: 40,
                                            }
                                       }
                                   }
                               }],
                            }]
                        };
                        // 为echarts对象加载数据
                        myChart.setOption(option);
                    </script>
                </div>
            </div>
    """

    temp_label = '''
        <div><span>&nbsp;</span></div>

        <div class="col-md-12">
            <div class="tabbable" id="tabs-957640">
                <ul class="nav nav-tabs">
                    <li class="active">
                        <a href="#panel-0" data-toggle="tab" style="Background-Color: #428bca; color: #fff;">全  部 ({})</a>
                    </li>
                    <li>
                        <a href="#panel-1" data-toggle="tab" style="Background-Color: #5cb85c; color: #fff;">成  功 ({})</a>
                    </li>
                    <li>
                        <a href="#panel-2" data-toggle="tab" style="Background-Color: #d9534f; color: #fff;">失  败 ({})</a>
                    </li>
                    <li>
                        <a href="#panel-3" data-toggle="tab" style="Background-Color: #5bc0de; color: #fff;">未验证 ({})</a>
                    </li>
                </ul>
            </div>
            <div class="tab-content">
                <div class="tab-pane active" id="panel-0">
                    <table class="table table-hover table-bordered">
    {}
                    </table>
                </div>


                <div class="tab-pane" id="panel-1">
                    <table class="table table-hover table-bordered">
    {}
                    </table>
                </div>


                <div class="tab-pane" id="panel-2">
                    <table class="table table-hover table-bordered">
    {}
                    </table>
                </div>


                <div class="tab-pane" id="panel-3">
                    <table class="table table-hover table-bordered">
    {}
                    </table>
                </div>
            </div>
        </div>
    </div>
    </body>
    </html>
    '''

    title_temp = '''
                <table class="table table-hover table-bordered" style="Background-Color:#dff0d8">
                    <thead>
                        <colgroup>
    {}
                        </colgroup>
                        <tr>
    {}
                        </tr>
                    </thead>
                    <tbody></tbody>
                </table>
     '''

    title_width = '''                         <col width='{}%'/>
    '''

    title = '''                         <td class="text-center"  style="Background-Color:#dff0d8">{}</td>
    '''

    table_data = '''                           <td class="text-center">{}</td>
    '''

    table_detail = '''                           <td class="text-center"><a href="javascript:showClassDetail('{}','{}', '{}')" class="detail" id = "{}">详细</a></td>
    '''

    detail_text = '''                   <tr class='hiddenRow' id="{}" >
                           <td colspan='{}'>
                               <div>
                                   <pre class="text-left" style="width:975;">{}</pre>
                               </div>
                           </td>
                       </tr> 
    '''

# 给pre的style加属性：width:975;
# 若pre换成p，加style="white-space:pre-wrap;"
# ontent="always" name="referrer"><meta name="theme-color" content="#ffffff"><meta name="description" content="全球领先的中文搜索引擎、致力于让网民更便捷地